==> dmachinemon-0.30.1.tar.gz: dmachinemon-0.30.1/tools/treeview/treeview-dotout.c <==
/*
 *  dmachinemon / a distributed machine monitor by dancer.
 *  Copyright (C) 2001 Junichi Uekawa
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * structure tree viewer: gives you a clue on tree structure of dmachinemon
 */
/* 
 * 2001 Sep 12
 * $Id: treeview-dotout.c,v 1.9 2002/05/22 11:11:17 dancer Exp $
 * $Log: treeview-dotout.c,v $
 * Revision 1.9  2002/05/22 11:11:17  dancer
 *  2002-05-22  Junichi Uekawa  <dancer@debian.org>
 *
 * +	* debian/rules: remove reference to VERSION.h
 * +
 * +	* tools/gtkiface/dmachinemon-gtkiface.c (main): use VERSION ditto.
 * +	* tools/treeview/treeview-dotout.c (main): ditto
 * +	* tools/treeview/treeview-3ddotout.c: ditto
 * +	* tools/treeview/treeview.c (main): ditto
 * +	* tools/htmloutput/dmachinemon-htmloutput.c (main): ditto
 * +	* tools/xmloutput/dmachinemon-xmloutput.c (main): ditto
 * +
 * +	* dmachinemon/dmachinemon-servent.c (main): use VERSION instead of DMACHINEMON_VERSION
 * +
 *  	* dmachinemon/dmachinemon-master.c (handle_clients): add debug level support.
 * +	use config.h and VERSION instead of DMACHINEMON_VERSION
 *
 *  	* dmachinemon/dmachinemon-servent.1: add --verbose.
 *
 * Revision 1.8  2002/05/21 04:49:55  dancer
 * make functions use const char* instead of char*
 *
 * Revision 1.7  2001/11/18 02:35:59  dancer
 * changing the rules
 *
 * Revision 1.6  2001/11/17 22:41:31  dancer
 * changed option processor to cdat.
 *
 * Revision 1.5  2001/11/07 06:32:42  dancer
 * library interface has been cleaned up, to prepend all functions with
 * "dm_" prefix.
 *
 * Revision 1.4  2001/10/02 10:54:18  dancer
 * added z axis
 *
 * Revision 1.3  2001/10/01 10:54:48  dancer
 * *** empty log message ***
 *
 * Revision 1.2  2001/10/01 10:40:18  dancer
 * working
 *
 * Revision 1.1  2001/10/01 07:24:26  dancer
 * working version.
 *
 */


#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <locale.h>
#include <pthread.h>

#define GRAPHICCENTER 40.0

#include "config.h"
#include "dmachinemon/dmachinemon.h"

#define PROGRAMNAME "treeview-dotout"

char * hostname ;
char * portnumber;

static void * 
malloc_with_error (int size)
{
  void * m = malloc (size);
  if (!m )
    {
      fprintf (stderr, PROGRAMNAME ": out of memory\n");
      exit (EXIT_FAILURE);
    }
  return m;
}

typedef struct machinetree
{
  char * myname;
  struct machinetree * nextlink;
  struct machinetree * downlink;
} machinetree;

/* getcar */
static char *
getcar (const char * ch)
{
  char * b = ch?strrchr(ch, ','):NULL;
  if (b) b++;
  return b?strdup(b):(ch?strdup(ch):NULL);
}

/* getcdr */
static char*
getcdr (const char * ch)
{
  char * s = ch?strdup(ch):NULL;
  char * b = s?strrchr (s, ','):NULL;
  if (b) 
    *b = 0;
  else 
    {
      free (s);
      s=NULL;
    }
  return s;
}

static void 
processinternal (machinetree * t, char * data)
{
  machinetree * mydata ;
  int found;
  char * buf = getcar (data);
  if (!data)
    return;

  found = 0;
  mydata = t -> downlink;
  while (mydata)
    {
      if (!strcmp(mydata -> myname, buf))
	{
	  found = 1 ;      
	  break;
	}
      mydata = mydata -> nextlink;
    }
  if (!found)
    {
      mydata = malloc(sizeof (machinetree));
      mydata -> downlink = NULL;
      mydata -> myname = buf;
      mydata -> nextlink = t -> downlink;
      t->downlink = mydata;
    }
  else
    free (buf);
  processinternal(mydata, getcdr(data));
  free (data);
}

static machinetree * 
build_local_route_tree (void)
{
  dm_machinelist * new = NULL;
  machinetree * treeptr = NULL;

  /* creating the tree */
  new = maininfo.machinedb;
  treeptr = malloc_with_error (sizeof(machinetree));
  treeptr -> myname = NULL;
  treeptr -> nextlink = treeptr -> downlink = NULL;

  while(new)
    {
      const char * buf;
      buf = dm_get_value_text(new, "Seen-By");
      if (buf)
	{
	  if (!treeptr->myname)
	    treeptr->myname = getcar (buf);
	  if (strchr (buf, ','))
	    processinternal (treeptr,getcdr(buf));
	}
      new = new -> next;
    }  
  return treeptr;  
}


static void access_master(void)
{
  int fd = dm_connect_to_host(hostname, atoi(portnumber));
  FILE* f=(fd!=-1)?fdopen (fd, "r"):NULL;
  
  if (!f)
    {
      fprintf (stderr, PROGRAMNAME ": cannot open server socket\n");
      exit(1);
    }

  if (dm_process_nodes_input (f, &maininfo, NULL, NULL))
    {
      fprintf (stderr, PROGRAMNAME ": could not handle input...\n");
      exit (1);
    }

  fclose(f);
}


/* elle is the allowed space. */
static void
display_local_route_tree(machinetree* m, const float x, const float y, float elle, const int have_parent_flag, float parentx, float parenty)
{
  int numbercount = 0 ;
  const int yoffset = 2.0;
  machinetree * mym = m;
  float xoffset  ;
  float myx ;
  while (mym)
    {
      numbercount ++;
      mym=mym->nextlink;
    }
  
  elle /= (float)numbercount;
  xoffset = elle;

  while (m)
    {
      if (!have_parent_flag)
	{
	  parentx=x+xoffset;
	  parenty=y;
	}
      
      myx = x+ xoffset;
      
      printf ("%f %f 0 %s %f %f 0\n", myx  - GRAPHICCENTER * 2.0, y, m->myname, parentx - GRAPHICCENTER * 2.0, parenty);
      
      if (m->downlink)
	{
	  display_local_route_tree(m->downlink, x + xoffset, y - yoffset, elle, 1, x+ xoffset, y);
	}
      
      xoffset = - xoffset;
      if ( xoffset > 0 )
	xoffset += 2 * elle;
      
      m=m->nextlink;
    }  
}

static void
processing_command (void)
{
  machinetree * m;
  
  access_master();
  m = build_local_route_tree();
  display_local_route_tree(m, 0, 0, GRAPHICCENTER, 0, 0, 0);
  return ;  
}

int main(int ac, char ** av)
{ 
  dm_commandoption cdat;
  setlocale(LC_TIME, "");

  dmachinemon_parse_options(ac, av, &cdat);
  if (!cdat.parenthostname)
    {
      fprintf (stderr, 
	       "treeview-dotout v. %s \n\n"
	       "Not enough command-line information specified\n\n"
	       ,VERSION
	       );
      dmachinemon_print_commandline_help();
      return 1;
    }

  hostname=cdat.parenthostname;
  portnumber=cdat.port_client;
  processing_command();  
  return 0;
}

==> dmachinemon_0.32.5.orig.tar.gz: dmachinemon-0.32.5/tools/treeview/treeview-dotout.c <==
/*
 *  dmachinemon / a distributed machine monitor by dancer.
 *  Copyright (C) 2001 Junichi Uekawa
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * structure tree viewer: gives you a clue on tree structure of dmachinemon
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <locale.h>
#include <pthread.h>

#define GRAPHICCENTER 40.0

#include "config.h"
#include "dmachinemon/dmachinemon.h"

#define PROGRAMNAME "treeview-dotout"

char * hostname ;
char * portnumber;

static void * 
malloc_with_error (int size)
{
  void * m = malloc (size);
  if (!m )
    {
      fprintf (stderr, PROGRAMNAME ": out of memory\n");
      exit (EXIT_FAILURE);
    }
  return m;
}

typedef struct machinetree
{
  char * myname;
  struct machinetree * nextlink;
  struct machinetree * downlink;
} machinetree;

/* getcar */
static char *
getcar (const char * ch)
{
  char * b = ch?strrchr(ch, ','):NULL;
  if (b) b++;
  return b?strdup(b):(ch?strdup(ch):NULL);
}

/* getcdr */
static char*
getcdr (const char * ch)
{
  char * s = ch?strdup(ch):NULL;
  char * b = s?strrchr (s, ','):NULL;
  if (b) 
    *b = 0;
  else 
    {
      free (s);
      s=NULL;
    }
  return s;
}

static void 
processinternal (machinetree * t, char * data)
{
  machinetree * mydata ;
  int found;
  char * buf = getcar (data);
  if (!data)
    return;

  found = 0;
  mydata = t -> downlink;
  while (mydata)
    {
      if (!strcmp(mydata -> myname, buf))
	{
	  found = 1 ;      
	  break;
	}
      mydata = mydata -> nextlink;
    }
  if (!found)
    {
      mydata = malloc(sizeof (machinetree));
      mydata -> downlink = NULL;
      mydata -> myname = buf;
      mydata -> nextlink = t -> downlink;
      t->downlink = mydata;
    }
  else
    free (buf);
  processinternal(mydata, getcdr(data));
  free (data);
}

static machinetree * 
build_local_route_tree (void)
{
  dm_machinelist * new = NULL;
  machinetree * treeptr = NULL;

  /* creating the tree */
  new = maininfo.machinedb;
  treeptr = malloc_with_error (sizeof(machinetree));
  treeptr -> myname = NULL;
  treeptr -> nextlink = treeptr -> downlink = NULL;

  while(new)
    {
      const char * buf;
      buf = dm_get_value_text(new, "Seen-By");
      if (buf)
	{
	  if (!treeptr->myname)
	    treeptr->myname = getcar (buf);
	  if (strchr (buf, ','))
	    processinternal (treeptr,getcdr(buf));
	}
      new = new -> next;
    }  
  return treeptr;  
}


static void
access_master(dm_commandoption * cdat)
{
  int fd = DNAS_connect_to_host(cdat, hostname, atoi(portnumber),1);
  FILE* f=(fd!=-1)?fdopen (fd, "r"):NULL;
  
  if (!f)
    {
      fprintf (stderr, PROGRAMNAME ": cannot open server socket\n");
      exit(1);
    }

  if (dm_process_nodes_input (f, &maininfo, NULL, NULL))
    {
      fprintf (stderr, PROGRAMNAME ": could not handle input...\n");
      exit (1);
    }

  fclose(f);
}


/* elle is the allowed space. */
static void
display_local_route_tree(machinetree* m, const float x, const float y, float elle, const int have_parent_flag, float parentx, float parenty)
{
  int numbercount = 0 ;
  const int yoffset = 2.0;
  machinetree * mym = m;
  float xoffset  ;
  float myx ;
  while (mym)
    {
      numbercount ++;
      mym=mym->nextlink;
    }
  
  elle /= (float)numbercount;
  xoffset = elle;

  while (m)
    {
      if (!have_parent_flag)
	{
	  parentx=x+xoffset;
	  parenty=y;
	}
      
      myx = x+ xoffset;
      
      printf ("%f %f 0 %s %f %f 0\n", myx  - GRAPHICCENTER * 2.0, y, m->myname, parentx - GRAPHICCENTER * 2.0, parenty);
      
      if (m->downlink)
	{
	  display_local_route_tree(m->downlink, x + xoffset, y - yoffset, elle, 1, x+ xoffset, y);
	}
      
      xoffset = - xoffset;
      if ( xoffset > 0 )
	xoffset += 2 * elle;
      
      m=m->nextlink;
    }  
}

static void
processing_command (dm_commandoption * cdat)
{
  machinetree * m;
  
  access_master(cdat);
  m = build_local_route_tree();
  display_local_route_tree(m, 0, 0, GRAPHICCENTER, 0, 0, 0);
  return ;  
}

int main(int ac, char ** av)
{ 
  dm_commandoption cdat;
  setlocale(LC_TIME, "");

  dmachinemon_parse_options(ac, av, &cdat);
  if (!cdat.parenthostname)
    {
      fprintf (stderr, 
	       "treeview-dotout v. %s \n\n"
	       "Not enough command-line information specified\n\n"
	       ,VERSION
	       );
      dmachinemon_print_commandline_help();
      return 1;
    }

  hostname=cdat.parenthostname;
  portnumber=cdat.port_client;
  processing_command(&cdat);  
  return 0;
}
